%% tclp__main.pl
%% Copyright (C) 2001,2002 INRIA Emmanuel Coquery

%% This library is free software; you can redistribute it and/or
%% modify it under the terms of the GNU Lesser General Public
%% License as published by the Free Software Foundation; either
%% version 2.1 of the License, or (at your option) any later version.

%% This library is distributed in the hope that it will be useful,
%% but WITHOUT ANY WARRANTY; without even the implied warranty of
%% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
%% Lesser General Public License for more details.

%% You should have received a copy of the GNU Lesser General Public
%% License along with this library; if not, write to the Free Software
%% Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%                                                                           %%
%%                                  TCLP                                     %%
%%                                                                           %%
%%         Emmanuel Coquery, projet Contraintes, INRIA Rocquencourt          %%
%%                                                                           %%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% $Header: /home/beaune/contraintes/coquery/cvs/contraintes/tclpchr/src/tclp__main.pl,v 1.25 2002/10/23 13:04:14 coquery Exp $

:- use_module(tclp).
:- use_module(tclp(tclp__utils)).
:- use_module(tclp(tclp__errors)).
:- use_module(tclp(tclp__files)).
:- use_module(tclp(sicstus_modules)).

:- use_module(library(lists)).

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% Process a file
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%% All the files to process
:- dynamic pl_files/1, typ_files/1.

pl_files([]).
typ_files([]).

%% add a prolog_file to all prolog files to type_check
add_pl_file(File) :-
        pl_files(CurrentFiles),
        append(CurrentFiles, [File], NewFiles),
        retractall(pl_files(_)),
        asserta(pl_files(NewFiles)).

%% add a typ_file to all typ files to load
add_typ_file(File) :-
        typ_files(CurrentFiles),
        append(CurrentFiles, [File], NewFiles),
        retractall(typ_files(_)),
        asserta(typ_files(NewFiles)).

:- dynamic stdlib_dir/1.

set_stdlib_dir(D) :-
        retractall(stdlib_dir(_)),
        assert(stdlib_dir(D)).

set_dialect(Prolog) :-
        (
          Prolog == sicstus
        ->
          enable_sicstus
        ;
          true
        ),
        tclp__directory(Dir),
        atom_concat(Dir,'/lib/',Dir2),
        atom_concat(Dir2,Prolog,Dir3),
        set_stdlib_dir(Dir3).

%% Process all arguments
process_args([]).
process_args([ '--prolog' , Prolog | Rest ]) :- !,
        verbose(debug_main,['processing --prolog ', Prolog, ' | ', Rest]),
        set_dialect(Prolog),
        process_args(Rest).
process_args([ '-l' | Rest ]) :- !,
        verbose(debug_main,['processing -l ', ' | ', Rest]),
        tclp__set_functor_labeling(true),
        process_args(Rest).
process_args([ '-nl' | Rest ]) :- !,
        verbose(debug_main,['processing -nl ', ' | ', Rest]),
        tclp__set_functor_labeling(false),
        process_args(Rest).
process_args([ '-I' , Dir | Rest ]) :- !,
        verbose(debug_main,['processing -I ', Dir, ' | ', Rest]),
        tclp__add_search_directory(Dir),!,
        process_args(Rest).
process_args([ '--stdlib' , StdLib | Rest ]) :-
        verbose(debug_main,['processing --stdlib ', StdLib, ' | ', Rest]),
        tclp__set_stdlib_name(StdLib),!,
        process_args(Rest).
process_args([ '-v', VerboseLevel | Rest ]) :-
        tclp__enable_verbose(VerboseLevel),!,
        process_args(Rest).
process_args([ '-nv', VerboseLevel | Rest ]) :-
        tclp__disable_verbose(VerboseLevel),!,
        process_args(Rest).
process_args([ '--help' | _ ]) :-
        print_usage, halt(1).
process_args([ '-i' | Rest ]) :- !,
        verbose(debug_main,['processing -i', ' | ', Rest]),
        tclp__set_inference(true),
        process_args(Rest).
process_args([ '--backward_compatibility' | Rest ]) :- !,
        verbose(debug_main,['processing --backward_compatibility', ' | ',
                            Rest]),
        tclp__declarations:enable_backward_compatibility,
        process_args(Rest).
process_args([ '--type_inference' | Rest ]) :- !,
        verbose(debug_main,['processing --type_inference', ' | ', Rest]),
        tclp__set_inference(true),
        process_args(Rest).
process_args([ '--type_check' | Rest ]) :- !,
        verbose(debug_main,['processing --type_check', ' | ', Rest]),
        tclp__set_inference(false),
        process_args(Rest).
process_args([ '--heuristic' | Rest ]) :- !,
        verbose(debug_main,['processing --heuristic', ' | ', Rest]),
        tclp__use_labeling_heuristic(true),
        process_args(Rest).
process_args([ '--look_ahead' , LA | Rest ]) :- !,
        verbose(debug_main,['processing --look_ahead ', LA, ' | ', Rest]),
        tclp__labeling_look_ahead(LA),
        process_args(Rest).
process_args([ '--strong_eq' | Rest ]) :- !,
        verbose(debug_main,['processing --strong_eq | ', Rest]),
        tclp__functor_types:set_equal_special_typing(true),
        process_args(Rest).
process_args([ Arg | Rest ]) :-
        verbose(debug_main,['processing ', Arg, ' | ', Rest]),
        chop_ext(Arg,'.pl',_),!,
        add_pl_file(Arg),
        process_args(Rest).
process_args([ Arg | Rest ]) :-
        verbose(debug_main,['processing ', Arg, ' | ', Rest]),
        chop_ext(Arg, '.typ',_),!,
        add_typ_file(Arg),
        process_args(Rest).
process_args([ Arg | _ ]) :-
        print_usage,
        print_error(['Illegal argument : ',Arg]).

%% Run the type checker according to the given arguments
run(Args) :-
        verbose(debug_main,'starting cmd arg parsing'),
        process_args(Args),
        verbose(debug_main,'cmd line ok'),
        once((stdlib_dir(_); set_dialect('ISO'))), % default dialect
        stdlib_dir(StdLibDir),
        tclp__add_search_directory(StdLibDir),
        verbose(actions,[ 'Standard type directory: ',StdLibDir]),
        tclp:stdlib(StdLib),
        verbose(actions,[ 'Loading standard type library (',StdLib,')' ]),
        once(chrono([ 'loading standard type library (',StdLib,')' ],
                    tclp__process_typ([StdLib]))),
        typ_files(TypFiles),
        (
          TypFiles \= []
        ->
          verbose(actions,['Loading : ',TypFiles]),
          once(chrono(['loading : ',TypFiles,' ...'],
                      tclp__process_typ(TypFiles)))
        ;
          true
        ),
        pl_files(PlFiles),
        tclp__process_pl(PlFiles),
        verbose(debug_main,'Finished')
        ;
        print_error('Failed').

%% Usage message
print_usage :-
        nl,write('usage:'),nl,
        write('  tclp [ --prolog <dialect> ] [ -I <dir> ] [ --stdlib <file> ]'),nl,
        write('       [ -v <sort> | -nv <sort> ] [ -i | --type_inference | --type_check ]'),nl,
        write('       [ -l | -nl ] [ --help ] <file> [ <files> ]'),nl,nl,
        write('--prolog <dialect>'),nl,
        write('    <dialect> is the name of a prolog dialect. This option causes TCLP to'),nl,
        write('    use type definitions for the corresponding dialect. Currently,'),nl,
        write('    available dialects are:'),nl,
        write('      - ISO: ISO Prolog (default)'),nl,
        write('      - gprolog: GNU Prolog'),nl,
        write('      - sicstus: SICStus Prolog'),nl,
        write('-I <dir>'),nl,
        write('    This options adds <dir> to the list of directories where TCLP'),nl,
        write('    looks for files.'),nl,
        write('--stdlib <file>'),nl,
        write('    Makes TCLP use <file> as the standard type library (for built-in'),nl,
        write('    predicates). By default TCLP will use the first file named'),nl,
        write('    "stdlib.typ" in the lookup directories (which contains the dialect'),nl,
        write('    directory "tclpdir/lib/dialect").'),nl,
        write('-v <sort>'),nl,
        write('    Enable verbose mode for <sort>, where <sort> can be either:'),nl,
        write('      - actions: prints the different actions of tclp, like, e.g.,'),nl,
        write('          loading a type library.'),nl,
        write('      - timing: prints CPU time used by TCLP for different actions.'),nl,
        write('      - defaults: output a message each time tclp encounters a term that'),nl,
        write('          has no defined type. This can be useful to understand some'),nl,
        write('          error messages.'),nl,
        write('      - types: prints the type of each infered predicate (this is'),nl,
        write('activated by default)'),nl,
        write('-nv <sort>'),nl,
        write('    Disables verbose mode for <sort>.'),nl,
        write('-i, --type_inference'), nl,
        write('    Enables predicate type inference (this is activated by default).'),nl,
        write('--type_check'),nl,
        write('    Disables predicate type inference.'),nl,
        write('--help'),nl,
        write('    Prints this message.'),nl,nl.

:- include(tclp('version.pl')).

%% Run the type checker according to the command line arguments
run :-
        (
          tclp__version(Version),
          write('%% tclp '),write(Version),nl,
          write('%% tclp directory: '),tclp__directory(D), write(D),nl
        ;
          true
        ),
        prolog_flag(argv, Args),
        catch(chrono('all',run(Args)),
              Exception,
              (
                (Exception \= tclp__error ->
                    print_exception(Exception);
                    write('! Failed'),nl),
                halt(1)
              )
             ),
        nl,
        halt(0)
        ;
        halt(2).

user:runtime_entry(start) :- run.
